﻿using System;
using HeuristicLab.Core;
using HeuristicLab.Random;

namespace HeuristicLab.Algorithms.PushGP.Generators
{
    public class NameGenerator
    {
        public const int DefaultRandomNameLength = 10;
        private static IRandom rand = new FastRandom();
        private static char[] chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890".ToCharArray();

        private int index = 0;

        public string GetNextString()
        {
            var length = index < chars.Length ? 1 : (int)(Math.Log(index, chars.Length)) + 1;
            var result = new char[length];
            var tmp = index;
            index++;

            for (var i = length; i > 0; i--)
            {
                var a = (int)Math.Pow(chars.Length, i - 1);
                var b = tmp / a;

                result[length - i] = chars[b];

                tmp -= a * b;
            }

            return new string(result);
        }

        public static string RandomName(int length = DefaultRandomNameLength)
        {
            var name = new char[length];

            for (var i = 0; i < length; i++)
            {
                name[i] = chars[rand.Next(chars.Length)];
            }

            return new string(name);
        }
    }
}
